# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::DataFactory::Mgmt::V2017_09_01_preview
  #
  # The Azure Data Factory V2 management API provides a RESTful set of web
  # services that interact with Azure Data Factory V2 services.
  #
  class ActivityRuns
    include MsRestAzure

    #
    # Creates and initializes a new instance of the ActivityRuns class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [DataFactoryManagementClient] reference to the DataFactoryManagementClient
    attr_reader :client

    #
    # List activity runs based on input filter conditions.
    #
    # @param resource_group_name [String] The resource group name.
    # @param factory_name [String] The factory name.
    # @param run_id [String] The pipeline run identifier.
    # @param start_time [DateTime] The start time of activity runs in ISO8601
    # format.
    # @param end_time [DateTime] The end time of activity runs in ISO8601 format.
    # @param status [String] The status of the pipeline run.
    # @param activity_name [String] The name of the activity.
    # @param linked_service_name [String] The linked service name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ActivityRun>] operation results.
    #
    def list_by_pipeline_run(resource_group_name, factory_name, run_id, start_time, end_time, status:nil, activity_name:nil, linked_service_name:nil, custom_headers:nil)
      first_page = list_by_pipeline_run_as_lazy(resource_group_name, factory_name, run_id, start_time, end_time, status:status, activity_name:activity_name, linked_service_name:linked_service_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param resource_group_name [String] The resource group name.
    # @param factory_name [String] The factory name.
    # @param run_id [String] The pipeline run identifier.
    # @param start_time [DateTime] The start time of activity runs in ISO8601
    # format.
    # @param end_time [DateTime] The end time of activity runs in ISO8601 format.
    # @param status [String] The status of the pipeline run.
    # @param activity_name [String] The name of the activity.
    # @param linked_service_name [String] The linked service name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_pipeline_run_with_http_info(resource_group_name, factory_name, run_id, start_time, end_time, status:nil, activity_name:nil, linked_service_name:nil, custom_headers:nil)
      list_by_pipeline_run_async(resource_group_name, factory_name, run_id, start_time, end_time, status:status, activity_name:activity_name, linked_service_name:linked_service_name, custom_headers:custom_headers).value!
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param resource_group_name [String] The resource group name.
    # @param factory_name [String] The factory name.
    # @param run_id [String] The pipeline run identifier.
    # @param start_time [DateTime] The start time of activity runs in ISO8601
    # format.
    # @param end_time [DateTime] The end time of activity runs in ISO8601 format.
    # @param status [String] The status of the pipeline run.
    # @param activity_name [String] The name of the activity.
    # @param linked_service_name [String] The linked service name.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_pipeline_run_async(resource_group_name, factory_name, run_id, start_time, end_time, status:nil, activity_name:nil, linked_service_name:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MaxLength': '90'" if !resource_group_name.nil? && resource_group_name.length > 90
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._\(\)]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._\(\)]+$$')).nil?
      fail ArgumentError, 'factory_name is nil' if factory_name.nil?
      fail ArgumentError, "'factory_name' should satisfy the constraint - 'MaxLength': '63'" if !factory_name.nil? && factory_name.length > 63
      fail ArgumentError, "'factory_name' should satisfy the constraint - 'MinLength': '3'" if !factory_name.nil? && factory_name.length < 3
      fail ArgumentError, "'factory_name' should satisfy the constraint - 'Pattern': '^[A-Za-z0-9]+(?:-[A-Za-z0-9]+)*$'" if !factory_name.nil? && factory_name.match(Regexp.new('^^[A-Za-z0-9]+(?:-[A-Za-z0-9]+)*$$')).nil?
      fail ArgumentError, 'run_id is nil' if run_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, 'start_time is nil' if start_time.nil?
      fail ArgumentError, 'end_time is nil' if end_time.nil?
      fail ArgumentError, "'linked_service_name' should satisfy the constraint - 'MaxLength': '260'" if !linked_service_name.nil? && linked_service_name.length > 260
      fail ArgumentError, "'linked_service_name' should satisfy the constraint - 'MinLength': '1'" if !linked_service_name.nil? && linked_service_name.length < 1
      fail ArgumentError, "'linked_service_name' should satisfy the constraint - 'Pattern': '^[A-Za-z0-9_][^<>*#.%&:\\+?/]*$'" if !linked_service_name.nil? && linked_service_name.match(Regexp.new('^^[A-Za-z0-9_][^<>*#.%&:\\+?/]*$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DataFactory/factories/{factoryName}/pipelineruns/{runId}/activityruns'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'factoryName' => factory_name,'runId' => run_id},
          query_params: {'api-version' => @client.api_version,'startTime' => start_time,'endTime' => end_time,'status' => status,'activityName' => activity_name,'linkedServiceName' => linked_service_name},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataFactory::Mgmt::V2017_09_01_preview::Models::ActivityRunsListResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ActivityRunsListResponse] operation results.
    #
    def list_by_pipeline_run_next(next_page_link, custom_headers:nil)
      response = list_by_pipeline_run_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_pipeline_run_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_pipeline_run_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_pipeline_run_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataFactory::Mgmt::V2017_09_01_preview::Models::ActivityRunsListResponse.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List activity runs based on input filter conditions.
    #
    # @param resource_group_name [String] The resource group name.
    # @param factory_name [String] The factory name.
    # @param run_id [String] The pipeline run identifier.
    # @param start_time [DateTime] The start time of activity runs in ISO8601
    # format.
    # @param end_time [DateTime] The end time of activity runs in ISO8601 format.
    # @param status [String] The status of the pipeline run.
    # @param activity_name [String] The name of the activity.
    # @param linked_service_name [String] The linked service name.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ActivityRunsListResponse] which provide lazy access to pages of the
    # response.
    #
    def list_by_pipeline_run_as_lazy(resource_group_name, factory_name, run_id, start_time, end_time, status:nil, activity_name:nil, linked_service_name:nil, custom_headers:nil)
      response = list_by_pipeline_run_async(resource_group_name, factory_name, run_id, start_time, end_time, status:status, activity_name:activity_name, linked_service_name:linked_service_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_pipeline_run_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
